package ca.tomahawk.robotbrain;

import android.content.Context;
import android.graphics.*;
import android.util.AttributeSet;
import android.view.View;

public class DrawView extends View 
{
	private final int NUMBER_OF_READINGS=18;  // 18 =  180 degrees/10 readings

    private Paint m_paint;
    private DataReadings[] m_readings = new DataReadings[NUMBER_OF_READINGS];
    private DataReadings lastReading = new DataReadings();

    public DrawView(Context context, AttributeSet attrs) 
    {
    	super( context, attrs );
        CreateView();
    }
    	 
    public DrawView(Context context, AttributeSet attrs, int defStyle) 
    {
    	super( context, attrs, defStyle );
        CreateView();
    }
    
    public DrawView(Context context)
    {
        super(context);
        CreateView();
    }
    
    public void CreateView( )
    {   	
        m_paint = new Paint();
        m_paint.setColor(Color.RED);
        m_paint.setStrokeWidth(2); 
        
        if( !isInEditMode() )
        {
	        for( int i=0;i<NUMBER_OF_READINGS;i++)
	        	m_readings[i] = new DataReadings();
        }
    }
    
    public void SetDataReading( int ang, int dist )
    {
    	int height = getHeight();
    	
    	int idx = (ang>0) ? ang/10 : 0;
    	m_readings[idx].SetAngleDistance( ang, (dist>height?height-10:dist) );
    }

    
    @Override 
    protected void onDraw(Canvas canvas) 
    {
    	int dist = 0;
    	int width = getWidth();
    	
        canvas.drawColor(Color.WHITE);   
        
        if( !isInEditMode() )
        {
        	int xdelta =  width / NUMBER_OF_READINGS;
	        for( int angle=0;angle<NUMBER_OF_READINGS;angle++)
	        {
	        	dist = m_readings[angle].GetDistance();
	        	if( dist != 0 )
	        	{
	        		canvas.drawCircle( (xdelta*angle+(xdelta/2)), m_readings[angle].GetDistance(), 5, m_paint );
	        		if( angle > 0 )
		        		canvas.drawLine( lastReading.GetAngle(), lastReading.GetDistance(), (xdelta*angle+(xdelta/2)), m_readings[angle].GetDistance(), m_paint );        	
	        		
        			lastReading.SetAngleDistance( (xdelta*angle+(xdelta/2)), m_readings[angle].GetDistance() );
	        	}
	        }
        }
    }
}
