package ca.tomahawk.robotbrain;

import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import com.android.future.usb.UsbAccessory;
import com.android.future.usb.UsbManager;

import android.os.Bundle;
import android.os.ParcelFileDescriptor;
import android.app.Activity;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.util.Log;



public class RobotBrainActivity extends Activity 
{
	private static final String  TAG = RobotBrainActivity.class.getSimpleName();
	private static final String  ACTION_USB_PERMISSION = "ca.tomahawk.robotbrain.action.USB_PERMISSION";
	
	private PendingIntent        mPermissionIntent;
	private boolean              mPermissionRequestPending;
	private UsbManager           mUsbManager;
	private UsbAccessory         mAccessory;
	private ParcelFileDescriptor mFileDescriptor;
	private FileInputStream      mInputStream;
	private FileOutputStream     mOutputStream;

	//private TextView 			 m_dbgValueTextView;
	private DrawView			 m_drawView;
	
	private static final byte DISTANCE_READING = 0x1;
	// private static final byte TARGET_PIN       = 0x0;
	
    private static final byte COMMAND_TEXT   = 0xF; 
    private static final byte TARGET_DEFAULT = 0xF;

	/** Called when the activity is first created. */ 
	@Override
	public void onCreate(Bundle savedInstanceState) 
	{
		super.onCreate(savedInstanceState);

		mUsbManager       = UsbManager.getInstance(this);
		mPermissionIntent = PendingIntent.getBroadcast(this, 0, new Intent(ACTION_USB_PERMISSION), 0);
		
		IntentFilter filter = new IntentFilter(ACTION_USB_PERMISSION);
		filter.addAction(UsbManager.ACTION_USB_ACCESSORY_DETACHED);
		registerReceiver(mUsbReceiver, filter);

		setContentView(R.layout.activity_robot_brain);
		//m_dbgValueTextView = (TextView) findViewById(R.id.dbgTextView);
		m_drawView = (DrawView)findViewById(R.id.drawView);
	}
	
	/**
	 * Called when the activity is resumed from its paused state and immediately
	 * after onCreate().
	 */
	@Override
	public void onResume()
	{
		super.onResume();

		if (mInputStream != null && mOutputStream != null)
			return;

		UsbAccessory[] accessories = mUsbManager.getAccessoryList();
		UsbAccessory accessory = (accessories == null ? null : accessories[0]);
		if (accessory != null) 
		{
			if (mUsbManager.hasPermission(accessory))
				openAccessory(accessory);
			else
			{
				synchronized (mUsbReceiver) {
					if (!mPermissionRequestPending) 
					{
						mUsbManager.requestPermission(accessory,mPermissionIntent);
						mPermissionRequestPending = true;
					}
				}
			}
		} 
		else
			Log.d(TAG, "mAccessory is null");
	}

	/** Called when the activity is paused by the system. */
	@Override
	public void onPause() 
	{
		super.onPause();
		closeAccessory();
	}

	/**
	 * Called when the activity is no longer needed prior to being removed from
	 * the activity stack.
	 */
	@Override
	public void onDestroy() 
	{
		super.onDestroy();
		unregisterReceiver(mUsbReceiver);
	}

	private final BroadcastReceiver mUsbReceiver = new BroadcastReceiver() {
		@Override
		public void onReceive(Context context, Intent intent) 
		{
			String action = intent.getAction();
			if (ACTION_USB_PERMISSION.equals(action)) 
			{
				synchronized (this) {
					UsbAccessory accessory = UsbManager.getAccessory(intent);
					if (intent.getBooleanExtra( UsbManager.EXTRA_PERMISSION_GRANTED, false))
						openAccessory(accessory);
					else
						Log.d(TAG, "permission denied for accessory "+ accessory);
					
					mPermissionRequestPending = false;
				}
			} 
			else if (UsbManager.ACTION_USB_ACCESSORY_DETACHED.equals(action)) 
			{
				UsbAccessory accessory = UsbManager.getAccessory(intent);
				if (accessory != null && accessory.equals(mAccessory)) 
					closeAccessory();
			}
		}
	};

	private void openAccessory(UsbAccessory accessory) 
	{
		mFileDescriptor = mUsbManager.openAccessory(accessory);
		if (mFileDescriptor != null) 
		{
			mAccessory = accessory;
			FileDescriptor fd = mFileDescriptor.getFileDescriptor();
			mInputStream  = new FileInputStream(fd);
			mOutputStream = new FileOutputStream(fd);
			
			Thread thread = new Thread(null, commRunnable, TAG);
			thread.start();
			
			Log.d(TAG, "accessory opened");
		} 
		else
			Log.d(TAG, "accessory open fail");
	}

	private void closeAccessory() 
	{
		try {
			if (mFileDescriptor != null)
				mFileDescriptor.close();
		} 
		catch (IOException e) {  } 
		finally {
			mFileDescriptor = null;
			mAccessory = null;
		}
	}

	Runnable commRunnable = new Runnable() {

		public void run() 
		{
			int ret = 0;
			byte[] buffer = new byte[255];

			while (ret >= 0) 
			{
				try {
					ret = mInputStream.read(buffer);
				} 
				catch (IOException e) {
					Log.e(TAG, "IOException : " + e.getMessage(), e);
					break;
				}
				
				switch ( buffer[0] ) 
				{ 
				case DISTANCE_READING:
					final int angle    = ((buffer[1] & 0xFF) << 8)  +  (buffer[2] & 0xFF);
					final int distance = ((buffer[3] & 0xFF) << 16) + ((buffer[4] & 0xFF) << 8) + (buffer[5] & 0xFF);
					
					final StringBuilder textBuilder = new StringBuilder();

					textBuilder.append( "Angle: "+angle );
					textBuilder.append( " Dist : "+distance );
				
					runOnUiThread( new Runnable() {
						public void run() 
						{
							DebugMessage( textBuilder.toString() );
							
							m_drawView.SetDataReading( angle, (distance*15) );
							m_drawView.invalidate();
						}
					});
					
					break;

				default:
					Log.d(TAG, "unknown msg: " + buffer[0]);
					DebugMessage( "unknown msg: " + buffer[0] );
					break;
				}
			}
		}
	};
	
	public void sendText( byte command, byte target, String text ) 
	{
        int textLength = text.length();
        byte[] buffer = new byte[3 + textLength];
        if (textLength <= 252) 
        {
            buffer[0] = command;
            buffer[1] = target;
            buffer[2] = (byte) textLength;
            byte[] textInBytes = text.getBytes();
            
            for (int x = 0; x < textLength; x++) 
                buffer[3 + x] = textInBytes[x];

            if ( mOutputStream != null ) 
            {
                try {
                    mOutputStream.write(buffer);
                } 
                catch (IOException e) {
                    Log.e(TAG, "write failed", e);
                }
            }
        }
    }
	
	private void DebugMessage( String msg )
	{
		//m_dbgValueTextView.setText( msg );

		// send a message back to the Arduino to acknowledge receipt
		sendText( COMMAND_TEXT, TARGET_DEFAULT, msg );
	}
}
